@@ -1,4 +1,4 @@
-/*      $Id: client.h 23704 2006-11-04 22:36:42Z olivier $
+/*      $Id: client.h 24169 2006-12-22 20:21:05Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -28,7 +28,8 @@
 #include <string.h>
 #include <signal.h>
 #include <unistd.h>
-#include <sys/types.h>
+
+#include <X11/X.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xmd.h>
@@ -37,6 +38,9 @@
 
 #include <glib.h>
 #include <gtk/gtk.h>
+#include <sys/time.h>
+#include <time.h>
+
 #include "screen.h"
 #include "misc.h"
 #include "hints.h"
@@ -79,6 +83,8 @@
                                          UPDATE_MAXIMIZE | \
                                          UPDATE_CACHE)
 
+#define STRUTS_SIZE                     12
+
 #ifndef CLIENT_MIN_VISIBLE
 #define CLIENT_MIN_VISIBLE              15
 #endif
@@ -184,16 +190,13 @@
 #define CLIENT_CAN_HIDE_WINDOW(c)       (!(c->transient_for) && \
                                          FLAG_TEST(c->xfwm_flags, XFWM_FLAG_HAS_HIDE) && \
                                          !FLAG_TEST(c->flags, CLIENT_FLAG_SKIP_TASKBAR))
-#define CLIENT_CAN_MAXIMIZE_WINDOW(c)   FLAG_TEST_ALL(c->xfwm_flags, XFWM_FLAG_HAS_MAXIMIZE | \
-                                                                     XFWM_FLAG_HAS_RESIZE | \
-                                                                     XFWM_FLAG_IS_RESIZABLE)
+#define CLIENT_CAN_MAXIMIZE_WINDOW(c)   (FLAG_TEST_AND_NOT(c->xfwm_flags, XFWM_FLAG_HAS_MAXIMIZE | \
+                                                                          XFWM_FLAG_HAS_RESIZE | \
+                                                                          XFWM_FLAG_IS_RESIZABLE, \
+                                                                          XFWM_FLAG_LEGACY_FULLSCREEN) && \
+                                         !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
 #define CLIENT_CAN_STICK_WINDOW(c)      (!(c->transient_for) && \
-                                         FLAG_TEST(c->xfwm_flags, XFWM_FLAG_HAS_STICK) && \
-                                         !FLAG_TEST(c->flags, CLIENT_FLAG_SKIP_TASKBAR))
-
-#define HINTS_ACCEPT_INPUT(wmhints)     (!(wmhints) || \
-                                         ((wmhints) && !(wmhints->flags & InputHint)) || \
-                                         ((wmhints) && (wmhints->flags & InputHint) && (wmhints->input)))
+                                         FLAG_TEST(c->xfwm_flags, XFWM_FLAG_HAS_STICK))
 
 typedef enum
 {
@@ -262,12 +265,14 @@ struct _Client
     int initial_layer;
     int ncmap;
     int button_pressed[BUTTON_COUNT];
-    int struts[12];
+    int struts[STRUTS_SIZE];
     gchar *name;
     Time user_time;
     unsigned long flags;
     unsigned long wm_flags;
     unsigned long xfwm_flags;
+    /* Timout for asynchronous icon update */
+    guint icon_timeout_id;
     /* Timout to manage blinking decorations for urgent windows */
     guint blink_timeout_id;
     /* Opacity for the compositor */
@@ -340,6 +345,8 @@ void                     clientHideAll                          (Client *,
                                                                  int);
 void                     clientClearAllShowDesktop              (ScreenInfo *);
 void                     clientToggleShowDesktop                (ScreenInfo *);
+void                     clientActivate                         (Client *,
+                                                                 Time);
 void                     clientClose                            (Client *);
 void                     clientKill                             (Client *);
 void                     clientEnterContextMenuState            (Client *);
