@@ -1,4 +1,4 @@
-/*      $Id: display.c 23704 2006-11-04 22:36:42Z olivier $
+/*      $Id: display.c 24107 2006-12-14 21:49:51Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #  include "config.h"
 #endif
 
+#include <X11/X.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/cursorfont.h>
@@ -424,6 +425,8 @@ myDisplayGrabServer (DisplayInfo *display)
 void
 myDisplayUngrabServer (DisplayInfo *display)
 {
+    g_return_if_fail (display);
+
     DBG ("entering myDisplayUngrabServer");
     display->xgrabcount = display->xgrabcount - 1;
     if (display->xgrabcount < 0)       /* should never happen */
@@ -706,14 +709,31 @@ myDisplayGetCurrentTime (DisplayInfo *display)
 {
     g_return_val_if_fail (display != NULL, (Time) CurrentTime);
 
+    TRACE ("myDisplayGetCurrentTime gives timestamp=%u", (unsigned int) display->current_time);
     return (Time) display->current_time;
 }
 
+Time
+myDisplayGetTime (DisplayInfo * display, Time timestamp)
+{
+    Time time;
+
+    time = timestamp;
+    if (time == (Time) CurrentTime)
+    {
+        time = getXServerTime (display);
+    }
+
+    TRACE ("myDisplayGetTime gives timestamp=%u", (unsigned int) time);
+    return (time);
+}
+
 Time
 myDisplayGetLastUserTime (DisplayInfo *display)
 {
     g_return_val_if_fail (display != NULL, (Time) CurrentTime);
 
+    TRACE ("myDisplayGetLastUserTime gives timestamp=%u", (unsigned int) display->last_user_time);
     return (Time) display->last_user_time;
 }
 
