@@ -1,4 +1,4 @@
-/*      $Id: netwm.c 23704 2006-11-04 22:36:42Z olivier $
+/*      $Id: netwm.c 24346 2007-01-10 20:50:18Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@
 #include "compositor.h"
 #include "display.h"
 #include "frame.h"
+#include "focus.h"
 #include "hints.h"
 #include "misc.h"
 #include "netwm.h"
@@ -334,23 +335,6 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
         }
     }
 
-    if ((first  == display_info->atoms[NET_WM_STATE_DEMANDS_ATTENTION]) ||
-        (second == display_info->atoms[NET_WM_STATE_DEMANDS_ATTENTION]))
-    {
-        if (action == NET_WM_STATE_ADD)
-        {
-            FLAG_SET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
-        }
-        else if (action == NET_WM_STATE_REMOVE)
-        {
-            FLAG_UNSET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
-        }
-        else if (action == NET_WM_STATE_TOGGLE)
-        {
-            FLAG_TOGGLE (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
-        }
-    }
-
     if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) ||
         (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) ||
         (first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]) ||
@@ -542,6 +526,34 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
         }
         frameDraw (c, TRUE);
     }
+
+    if ((first  == display_info->atoms[NET_WM_STATE_DEMANDS_ATTENTION]) ||
+        (second == display_info->atoms[NET_WM_STATE_DEMANDS_ATTENTION]))
+    {
+        if ((action == NET_WM_STATE_ADD) && !FLAG_TEST (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION))
+        {
+            /* Do not apply NET_WM_STATE_DEMANDS_ATTENTION if client is already focused */
+            if (c != clientGetFocusOrPending ())
+            {
+                FLAG_SET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
+                clientSetNetState (c);
+            }
+        }
+        else if ((action == NET_WM_STATE_REMOVE) && FLAG_TEST (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION))
+        {
+            FLAG_UNSET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
+            clientSetNetState (c);
+        }
+        else if (action == NET_WM_STATE_TOGGLE)
+        {
+            /* Do not apply NET_WM_STATE_DEMANDS_ATTENTION if client is already focused */
+            if (c != clientGetFocusOrPending () || !FLAG_TEST (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION))
+            {
+                FLAG_TOGGLE (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
+                clientSetNetState (c);
+            }
+        }
+    }
 }
 
 void
@@ -732,7 +744,7 @@ clientUpdateFullscreenState (Client * c)
          */
         myScreenGrabPointer (c->screen_info, EnterWindowMask, None, myDisplayGetCurrentTime (display_info));
         clientConfigure (c, &wc, CWX | CWY | CWWidth | CWHeight, CFG_FORCE_REDRAW);
-        myScreenUngrabPointer (c->screen_info, myDisplayGetCurrentTime (display_info));
+        myScreenUngrabPointer (c->screen_info);
     }
     else
     {
@@ -922,24 +934,56 @@ clientSetNetClientList (ScreenInfo * screen_info, Atom a, GList * list)
     }
 }
 
-void
+gboolean
+clientValidateNetStrut (Client * c)
+{
+    ScreenInfo *screen_info;
+    gboolean valid;
+    int max_value;
+    int i;
+
+    g_return_val_if_fail (c != NULL, TRUE);
+    TRACE ("entering clientValidateNetStrut for \"%s\" (0x%lx)", c->name, c->window);
+    screen_info = c->screen_info;
+    max_value = MIN (screen_info->width, screen_info->height) / 4;
+    valid = TRUE;
+
+    for (i = 0; i < 4; i++)
+    {
+        if (c->struts[i] > max_value)
+        {
+           g_warning ("Strut value for application window 0x%lx changed from %d to %d", c->window, c->struts[i], max_value);
+           c->struts[i] = max_value;
+           valid = FALSE;
+        }
+    }
+
+    return valid;
+}
+
+gboolean
 clientGetNetStruts (Client * c)
 {
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
+    unsigned long old_flags, new_flags;
+    int old_struts[STRUTS_SIZE];
     gulong *struts;
     int nitems;
     int i;
 
-    g_return_if_fail (c != NULL);
+    g_return_val_if_fail (c != NULL, FALSE);
     TRACE ("entering clientGetNetStruts for \"%s\" (0x%lx)", c->name, c->window);
 
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
     struts = NULL;
 
-    for (i = 0; i < 12; i++)
+    /* Save old values */
+    old_flags = c->flags & (CLIENT_FLAG_HAS_STRUT | CLIENT_FLAG_HAS_STRUT_PARTIAL);
+    for (i = 0; i < STRUTS_SIZE; i++)
     {
+        old_struts[i] = c->struts[i];
         c->struts[i] = 0;
     }
     FLAG_UNSET (c->flags, CLIENT_FLAG_HAS_STRUT);
@@ -947,24 +991,32 @@ clientGetNetStruts (Client * c)
 
     if (getCardinalList (display_info, c->window, NET_WM_STRUT_PARTIAL, &struts, &nitems))
     {
-        if (nitems != 12)
+        if (nitems != STRUTS_SIZE)
         {
             if (struts)
             {
                 XFree (struts);
             }
-            return;
+            /* Restore old values */
+            if (old_flags)
+            {
+                FLAG_SET (c->flags, old_flags);
+                for (i = 0; i < STRUTS_SIZE; i++)
+                {
+                    c->struts[i] = old_struts[i];
+                }
+            }
+            return FALSE;
         }
 
         FLAG_SET (c->flags, CLIENT_FLAG_HAS_STRUT);
         FLAG_SET (c->flags, CLIENT_FLAG_HAS_STRUT_PARTIAL);
-        for (i = 0; i < 12; i++)
+        for (i = 0; i < STRUTS_SIZE; i++)
         {
             c->struts[i] = (int) struts[i];
         }
 
         XFree (struts);
-        workspaceUpdateArea (c->screen_info);
     }
     else if (getCardinalList (display_info, c->window, NET_WM_STRUT, &struts, &nitems))
     {
@@ -974,7 +1026,16 @@ clientGetNetStruts (Client * c)
             {
                 XFree (struts);
             }
-            return;
+            /* Restore old values */
+            if (old_flags)
+            {
+                FLAG_SET (c->flags, old_flags);
+                for (i = 0; i < STRUTS_SIZE; i++)
+                {
+                    c->struts[i] = old_struts[i];
+                }
+            }
+            return FALSE;
         }
 
         FLAG_SET (c->flags, CLIENT_FLAG_HAS_STRUT);
@@ -982,7 +1043,7 @@ clientGetNetStruts (Client * c)
         {
             c->struts[i] = (int) struts[i];
         }
-        for (i = 4; i < 12; i++)
+        for (i = 4; i < STRUTS_SIZE; i++)
         {
             c->struts[i] = 0;
         }
@@ -995,8 +1056,33 @@ clientGetNetStruts (Client * c)
             c->screen_info->height;
 
         XFree (struts);
-        workspaceUpdateArea (c->screen_info);
     }
+
+    if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_STRUT))
+    {
+        clientValidateNetStrut (c);
+    }
+
+    /* check for a change in struts flags */
+    new_flags = c->flags & (CLIENT_FLAG_HAS_STRUT | CLIENT_FLAG_HAS_STRUT_PARTIAL);
+    if (old_flags != new_flags)
+    {
+        return TRUE;
+    }
+
+    /* Flags haven't changed, check values */
+    if (new_flags)
+    {
+        for (i = 0; i < STRUTS_SIZE; i++)
+        {
+            if (old_struts[i] != c->struts[i])
+            {
+                return TRUE;
+            }
+        }
+    }
+
+    return FALSE;
 }
 
 void
@@ -1076,7 +1162,7 @@ clientWindowType (Client * c)
             FLAG_UNSET (c->xfwm_flags,
                 XFWM_FLAG_HAS_BORDER |  XFWM_FLAG_HAS_MOVE |
                 XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE |
-                XFWM_FLAG_HAS_MENU | XFWM_FLAG_HAS_STICK);
+                XFWM_FLAG_HAS_MENU);
         }
         else if (c->type_atom == display_info->atoms[NET_WM_WINDOW_TYPE_TOOLBAR])
         {
@@ -1086,8 +1172,7 @@ clientWindowType (Client * c)
             FLAG_SET (c->flags,
                 CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_SKIP_TASKBAR);
             FLAG_UNSET (c->xfwm_flags,
-                XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE |
-                XFWM_FLAG_HAS_STICK);
+                XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE);
         }
         else if (c->type_atom == display_info->atoms[NET_WM_WINDOW_TYPE_MENU])
         {
@@ -1101,8 +1186,7 @@ clientWindowType (Client * c)
             FLAG_SET (c->flags,
                 CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_SKIP_TASKBAR);
             FLAG_UNSET (c->xfwm_flags,
-                XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE |
-                XFWM_FLAG_HAS_STICK);
+                XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE);
         }
         else if (c->type_atom == display_info->atoms[NET_WM_WINDOW_TYPE_DIALOG])
         {
@@ -1134,7 +1218,7 @@ clientWindowType (Client * c)
             FLAG_UNSET (c->xfwm_flags,
                 XFWM_FLAG_HAS_BORDER | XFWM_FLAG_HAS_HIDE |
                 XFWM_FLAG_HAS_MENU | XFWM_FLAG_HAS_MOVE |
-                XFWM_FLAG_HAS_RESIZE | XFWM_FLAG_HAS_STICK);
+                XFWM_FLAG_HAS_RESIZE);
         }
     }
     else
