@@ -1,4 +1,4 @@
-/*      $Id: stacking.c 23128 2006-09-11 20:39:40Z olivier $
+/*      $Id: stacking.c 24107 2006-12-14 21:49:51Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -255,7 +255,7 @@ clientAtPosition (ScreenInfo *screen_info, int x, int y, Client * exclude)
         if ((frameX (c2) < x) && (frameX (c2) + frameWidth (c2) > x)
             && (frameY (c2) < y) && (frameY (c2) + frameHeight (c2) > y))
         {
-            if (clientSelectMask (c2, 0, WINDOW_REGULAR_FOCUSABLE)
+            if (clientSelectMask (c2, INCLUDE_SKIP_PAGER | INCLUDE_SKIP_TASKBAR, WINDOW_REGULAR_FOCUSABLE)
                 && (c2 != exclude))
             {
                 c = c2;
